
\name{e04uf}
\alias{e04uf}

\title{e04uf: Minimum, function of several variables, sequential QP method, nonlinear 
constraints, using function values and optionally first derivatives (reverse 
communication, comprehensive)}

\description{
e04uf is designed to minimize an arbitrary smooth function subject to 
constraints (which may include simple bounds on the variables, linear constraints 
and smooth nonlinear constraints) using a sequential quadratic programming 
(SQP) method. As many first derivatives as possible should be supplied 
by you; any unspecified derivatives are approximated by finite differences. 
It is not intended for large sparse problems.

e04uf may also be used for unconstrained, bound-constrained and linearly 
constrained optimization.

e04uf uses \bold{reverse communication} for evaluating the objective function, 
the nonlinear constraint functions and any of their derivatives.


}

\usage{
e04uf(irevcm, nclin, a, bl, bu, iter, istate, c, cjac, clamda, objf, objgrd, r, x, iwork, work, cwsav, lwsav, iwsav, rwsav, optlist,
      n = nrow(objgrd),
      ncnln = nrow(c))
}

\arguments{

\item{irevcm}{integer

Must be set to \eqn{{0}}{0}.

\bold{must remain unchanged}, unless you wish to terminate the solution 
to the current problem. In this case irevcm may be set to a negative value 
and then e04uf will take a final exit with ifail set to this value of 
irevcm.


}
\item{nclin}{integer

\eqn{{n}_{{L}}}{n_L}, the number of general linear constraints.


}
\item{a}{double array

The \eqn{{i}}{i}th row of the array a must contain the \eqn{{i}}{i}th row 
of the matrix \eqn{{A}_{{L}}}{A_L} of general linear constraints in eqn1. 
That is, the \eqn{{i}}{i}th row contains the coefficients of the \eqn{{i}}{i}th 
general linear constraint for \eqn{{i}={1} \ldots nclin}{i=1  . . .  nclin}.


}
\item{bl}{double array}
\item{bu}{double array

Bl must contain the lower bounds and bu the upper bounds, for all the constraints 
in the following order. The first \eqn{{n}}{n} elements of each array must 
contain the bounds on the variables, the next \eqn{{n}_{{L}}}{n_L} elements 
the bounds for the general linear constraints (if any) and the next \eqn{{n}_{{N}}}{n_N} 
elements the bounds for the general nonlinear constraints (if any). To 
specify a nonexistent lower bound (i.e., \eqn{{l}_{{j}} = { - {\infty}}}{l_j =  -  infinity}), 
set \eqn{bl[{j}] \leq - {bigbnd}}{bl[j]  <=  - bigbnd}, and to specify a nonexistent 
upper bound (i.e., \eqn{{u}_{{j}} = { + {\infty}}}{u_j =  +  infinity}), set 
\eqn{bu[{j}] \geq {bigbnd}}{bu[j]  >=  bigbnd}; the default value of \eqn{{bigbnd}}{bigbnd} 
is \eqn{{10}^{{20}}}{10^20}, but this may be changed by the optional argument 
infiniteboundsize. To specify the \eqn{{j}}{j}th constraint as an \emph{equality}, 
set \eqn{bl[{j}] = bu[{j}] = {\beta}}{bl[j] = bu[j] = \beta}, say, where \eqn{\mathrm{abs}({\beta}) 
< {bigbnd}}{abs(\beta) < bigbnd}.


}
\item{iter}{integer

Must remain unchanged from a previous call to e04uf.


}
\item{istate}{integer array

Need not be set if the (default) optional argument coldstart is used.


}
\item{c}{double array

Need not be set.

If \eqn{irevcm={4}, {6}}{irevcm=4, 6} and \eqn{needc[{i}] > {0}}{needc[i] > 0}, 
\eqn{c[{i}]}{c[i]} must contain the value of the \eqn{{i}}{i}th constraint 
at \eqn{{x}}{x}. The remaining elements of c, corresponding to the non-positive 
elements of needc, are ignored.


}
\item{cjac}{double array

In general, cjac need not be initialized before the call to e04uf. However, 
if the optional argument \eqn{derivativelevel={2}, {3}}{derivativelevel=2, 3}, 
you may optionally set the constant elements of cjac. Such constant elements 
need not be re-assigned on subsequent intermediate exits.

If \eqn{irevcm={5}, {6}}{irevcm=5, 6} and \eqn{needc[{i}] > {0}}{needc[i] > 0}, 
the \eqn{{i}}{i}th row of cjac must contain the available elements of the 
vector \eqn{{\nabla}{c}_{{i}}}{\nablac_i} given by \deqn{{\nabla}{c}_{{i}} = \left(\frac{{ 
\partial {c}_{{i}}}}{{ \partial {x}_{{1}}}} , \frac{{ \partial {c}_{{i}}}}{{ \partial {x}_{{2}}}} 
, \ldots , \frac{{ \partial {c}_{{i}}}}{{ \partial {x}_{{n}}}}\right)^{T}\mbox{,}}{\nablac_i = (( \partial c_i)/( \partial x_1) , ( \partial c_i)/( \partial x_2) ,  . . .  , ( \partial c_i)/( \partial x_n))^T,} 
where \eqn{\frac{{ \partial {c}_{{i}}}}{{ \partial {x}_{{j}}}}}{( \partial c_i)/( \partial x_j)} 
is the partial derivative of the \eqn{{i}}{i}th constraint with respect 
to the \eqn{{j}}{j}th variable, evaluated at the point \eqn{{x}}{x}. The 
remaining rows of cjac, corresponding to non-positive elements of needc, 
are ignored.


}
\item{clamda}{double array

Need not be set if the (default) optional argument coldstart is used.


}
\item{objf}{double

Need not be set.

If \eqn{irevcm={1}, {3}}{irevcm=1, 3}, objf must be set to the value of 
the objective function at \eqn{{x}}{x}.


}
\item{objgrd}{double array

Need not be set.

If \eqn{irevcm={2}, {3}}{irevcm=2, 3}, objgrd must contain the available 
elements of the gradient evaluated at \eqn{{x}}{x}.


}
\item{r}{double array

Need not be initialized if the (default) optional argument coldstart is 
used.


}
\item{x}{double array

An initial estimate of the solution.


}
\item{iwork}{integer array




}
\item{work}{double array




}
\item{cwsav}{string array}{string array}
\item{lwsav}{boolean array}
\item{iwsav}{integer array}
\item{rwsav}{double array

The arrays lwsav, iwsav, rwsav and cwsav \bold{must not} be altered between calls to any of the functions e04wb, e04uf, e04ud e04ue.
}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Central Difference Interval} \tab \emph{double} \tab Default values are computed \cr
\code{Cold Start} \tab \emph{} \tab Default \cr
\code{Warm Start} \tab \emph{} \tab  \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.01}}{ = 0.01} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Derivative Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {3}}{ = 3} \cr
\code{Difference Interval} \tab \emph{double} \tab Default values are computed \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Function Precision} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.9}}}{ = \epsilon^0.9} \cr
\code{Hessian} \tab \emph{} \tab Default \eqn{= {NO}}{= NO} \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Infinite Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}({bigbnd}, {10}^{{20}})}{ = max(bigbnd, 10^20)} \cr
\code{Line Search Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{Linear Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Nonlinear Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.33}}}{ = \epsilon^0.33} or \eqn{\sqrt{{\epsilon}}}{sqrt(\epsilon)} \cr
\code{List} \tab \emph{} \tab  \cr
\code{Nolist} \tab \emph{} \tab  \cr
\code{Major Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{3}\left({n} + {n}_{{L}}\right) + {10}{n}_{{N}}})}{ = max(50, 3(n + n_L) + 10n_N)} \cr
\code{Iteration Limit} \tab \emph{} \tab  \cr
\code{Iters} \tab \emph{} \tab  \cr
\code{Itns} \tab \emph{} \tab  \cr
\code{Major Print Level} \tab \emph{integer} \tab  \cr
\code{Major Print Level} \tab \emph{integer} \tab  \cr
\code{Print Level} \tab \emph{} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Print Level} \tab \emph{} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Minor Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{3}\left({n} + {n}_{{L}} + {n}_{{N}}\right)})}{ = max(50, 3(n + n_L + n_N))} \cr
\code{Minor Print Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Monitoring File} \tab \emph{integer} \tab Default \eqn{\mbox{} = { - {1}}}{ =  - 1} \cr
\code{Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}_{{r}}^{{0.8}}}{ = \epsilon_r^0.8} \cr
\code{Start Objective Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Objective Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Start Constraint Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Constraint Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Step Limit} \tab \emph{double} \tab Default \eqn{\mbox{} = {2.0}}{ = 2.0} \cr
\code{Verify Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Verify} \tab \emph{} \tab  \cr
\code{Verify Constraint Gradients} \tab \emph{} \tab  \cr
\code{Verify Gradients} \tab \emph{} \tab  \cr
\code{Verify Objective Gradients} \tab \emph{} \tab  \cr
}
}
\item{n}{integer: \strong{default} = nrow(objgrd)

\eqn{{n}}{n}, the number of variables.


}
\item{ncnln}{integer: \strong{default} = nrow(c)

\eqn{{n}_{{N}}}{n_N}, the number of nonlinear constraints.


}
}

\details{
R interface to the NAG Fortran routine E04UFF.}

\value{

\item{IREVCM}{integer

Specifies what values the calling program must assign to arguments of e04uf 
before re-entering the function. 

\eqn{irevcm = {1}}{irevcm = 1}: Set objf to the value of the objective 
function \eqn{{F}\left({x}\right)}{F(x)}.

\eqn{irevcm = {2}}{irevcm = 2}: Set \eqn{objgrd[<{j}]}{objgrd[<j]} to the 
value \eqn{\frac{{ \partial {F}}}{{ \partial {x}_{{j}}}}}{( \partial F)/( \partial x_j)} if available 
for \eqn{{j}={1} \ldots {n}}{j=1  . . .  n}.

\eqn{irevcm = {3}}{irevcm = 3}: Set objf and \eqn{objgrd[{j}]}{objgrd[j]} 
as for \eqn{irevcm = {1}}{irevcm = 1} and \eqn{irevcm = {2}}{irevcm = 2}.

\eqn{irevcm 
= {4}}{irevcm = 4}: Set \eqn{c[{i}]}{c[i]} to the value of the constraint 
function \eqn{{c}_{{i}}\left({x}\right)}{c_i(x)}, for each \eqn{{i}}{i} 
such that \eqn{needc[{i}] > {0}}{needc[i] > 0}.

\eqn{irevcm = {5}}{irevcm = 5}: Set \eqn{cjac[{i}, {j}]}{cjac[i, j]} to 
the value \eqn{\frac{{ \partial {c}_{{i}}}}{{ \partial {x}_{{j}}}}}{( \partial c_i)/( \partial x_j)} 
if available, for each \eqn{{i}}{i} such that \eqn{needc[{i}] > {0}}{needc[i] > 0} 
and \eqn{{j} = {1} , {2} , \ldots , {n}}{j = 1 , 2 ,  . . .  , n}.

\eqn{irevcm = {6}}{irevcm = 6}: Set \eqn{c[{i}]}{c[i]} and \eqn{cjac[{i}, 
{j}]}{cjac[i, j]} as for \eqn{irevcm = {4}}{irevcm = 4} and \eqn{irevcm 
= {5}}{irevcm = 5}.



\eqn{irevcm = {0}}{irevcm = 0}.


}
\item{ITER}{integer

The number of major iterations performed.


}
\item{ISTATE}{integer array

The status of the constraints in the QP working set at the point returned 
in x. The significance of each possible value of \eqn{istate[{j}]}{istate[j]} 
is as follows:


}
\item{C}{double array

If \eqn{ncnln > {0}}{ncnln > 0}, \eqn{c[{i}]}{c[i]} contains the value 
of the \eqn{{i}}{i}th nonlinear constraint function \eqn{{c}_{{i}}}{c_i} 
at the final iterate for \eqn{{i}={1} \ldots ncnln}{i=1  . . .  ncnln}.


}
\item{CJAC}{double array

If \eqn{ncnln > {0}}{ncnln > 0}, cjac contains the Jacobian matrix of the 
nonlinear constraint functions at the final iterate, i.e., \eqn{cjac[{i}, 
{j}]}{cjac[i, j]} contains the partial derivative of the \eqn{{i}}{i}th 
constraint function with respect to the \eqn{{j}}{j}th variable for \eqn{{j}={1} 
\ldots n}{j=1  . . .  n} for \eqn{{i}={1} \ldots ncnln}{i=1  . . .  ncnln}.


}
\item{CLAMDA}{double array

The values of the QP multipliers from the last QP subproblem. \eqn{clamda[{j}]}{clamda[j]} 
should be non-negative if \eqn{istate[{j}] = {1}}{istate[j] = 1} and non-positive 
if \eqn{istate[{j}] = {2}}{istate[j] = 2}.


}
\item{OBJF}{double

The value of the objective function at the final iterate.


}
\item{OBJGRD}{double array

The gradient of the objective function at the final iterate (or its finite 
difference approximation).


}
\item{R}{double array

If \eqn{hessian=\code{NO}}{hessian=NO}, r contains the upper triangular 
Cholesky factor \eqn{{R}}{R} of \eqn{{Q}^{T}\tilde{{H}}{Q}}{Q^TtildeHQ}, 
an estimate of the transformed and reordered Hessian of the Lagrangian 
at \eqn{{x}}{x} (see eqn6 in the optional parameter description in the 
Fortran Library documentation).


}
\item{X}{double array

The point \eqn{{x}}{x} at which the objective function, constraint functions 
or their derivatives are to be evaluated.

The final estimate of the solution.


}
\item{NEEDC}{integer array

If \eqn{irevcm \geq {4}}{irevcm  >=  4}, needc specifies the indices of the 
elements of c and/or cjac that must be assigned. If \eqn{needc[{i}] > {0}}{needc[i] > 0}, 
then the \eqn{{i}}{i}th element of c and/or the available elements of the 
\eqn{{i}}{i}th row of cjac must be evaluated at \eqn{{x}}{x}.


}
\item{IWORK}{integer array


}
\item{WORK}{double array

The amounts of workspace provided and required may be (by default for e04uf) output on the current advisory message unit (as defined by x04ab).  As an alternative to computing liwork and lwork from the formulae given above, you may prefer to obtain appropriate values from the output of a preliminary run with liwork and lwork set to \eqn{{1}}{1}.  (e04uf will then terminate with \eqn{\mathrm{ifail} =9}{ifail =9}.)
}
\item{CWSAV}{string array

The arrays lwsav, iwsav, rwsav and cwsav \bold{must not} be altered between calls to any of the functions e04wb, e04uf, e04ud e04ue.
}{string array

The arrays lwsav, iwsav, rwsav and cwsav \bold{must not} be altered between calls to any of the functions e04wb, e04uf, e04ud e04ue.
}
\item{LWSAV}{boolean array

The arrays lwsav, iwsav, rwsav and cwsav \bold{must not} be altered between calls to any of the functions e04wb, e04uf, e04ud e04ue.
}
\item{IWSAV}{integer array

The arrays lwsav, iwsav, rwsav and cwsav \bold{must not} be altered between calls to any of the functions e04wb, e04uf, e04ud e04ue.
}
\item{RWSAV}{double array

The arrays lwsav, iwsav, rwsav and cwsav \bold{must not} be altered between calls to any of the functions e04wb, e04uf, e04ud e04ue.
}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04uff.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
iwork <- as.matrix(mat.or.vec(0, 0))
work <- as.matrix(mat.or.vec(0, 0))
cwsav <- as.matrix(mat.or.vec(0, 0))
lwsav <- as.matrix(mat.or.vec(0, 0))
iwsav <- as.matrix(mat.or.vec(0, 0))
rwsav <- as.matrix(mat.or.vec(0, 0))

irevcm <- 0

nclin <- 1

a <- matrix(c(1, 1, 1, 1), nrow = 1, ncol = 4, byrow = TRUE)



bl <- matrix(c(1, 1, 1, 1, -1e+25, -1e+25, 25), nrow = 7, 
    ncol = 1, byrow = TRUE)



bu <- matrix(c(5, 5, 5, 5, 20, 40, 1e+25), nrow = 7, 
    ncol = 1, byrow = TRUE)



iter <- 0

istate <- as.matrix(mat.or.vec(7, 1))

c <- matrix(c(0, 0), nrow = 2, ncol = 1, byrow = TRUE)



cjac <- matrix(c(0, 0, 0, 0, 0, 0, 0, 0), nrow = 2, 
    ncol = 4, byrow = TRUE)



clamda <- as.matrix(mat.or.vec(7, 1))

objf <- 0

objgrd <- as.matrix(mat.or.vec(4, 1))

r <- as.matrix(mat.or.vec(4, 4))

x <- matrix(c(1, 5, 5, 1), nrow = 4, ncol = 1, byrow = TRUE)



iwork <- as.matrix(mat.or.vec(17, 1))

work <- as.matrix(mat.or.vec(192, 1))

if (ifail == 0) {
    
    ans <- e04uf(irevcm, nclin, a, bl, bu, iter, istate, c, cjac, 
        clamda, objf, objgrd, r, x, iwork, work, cwsav, lwsav, 
        iwsav, rwsav, optlist)
    irevcm <- ans$IREVCM
    iter <- ans$ITER
    istate <- ans$ISTATE
    c <- ans$C
    cjac <- ans$CJAC
    clamda <- ans$CLAMDA
    objf <- ans$OBJF
    objgrd <- ans$OBJGRD
    r <- ans$R
    x <- ans$X
    needc <- ans$NEEDC
    iwork <- ans$IWORK
    work <- ans$WORK
    cwsav <- ans$CWSAV
    lwsav <- ans$LWSAV
    iwsav <- ans$IWSAV
    rwsav <- ans$RWSAV
    ifail <- ans$IFAIL
    while (irevcm > 0) {
        if (irevcm == 1 || irevcm == 3) {
            
            objf <- x[1] \%*\% x[4] \%*\% (x[1] + x[2] + x[3]) + 
                x[3]
            
        }
        if (irevcm == 2 || irevcm == 3) {
            
            objgrd[1] <- x[4] \%*\% (2 \%*\% x[1] + x[2] + x[3])
            
            objgrd[2] <- x[1] \%*\% x[4]
            
            objgrd[3] <- x[1] \%*\% x[4] + 1
            
            objgrd[4] <- x[1] \%*\% (x[1] + x[2] + x[3])
            
        }
        if (irevcm == 4 || irevcm == 6) {
            
            if (needc[1] > 0) {
                
                c[1] <- x[1]^2 + x[2]^2 + x[3]^2 + x[4]^2
                
            }
            if (needc[2] > 0) {
                
                c[2] <- x[1] \%*\% x[2] \%*\% x[3] \%*\% x[4]
                
            }
        }
        if (irevcm == 5 || irevcm == 6) {
            
            if (needc[1] > 0) {
                
                cjac[1, 1] <- 2 \%*\% x[1]
                
                cjac[1, 2] <- 2 \%*\% x[2]
                
                cjac[1, 3] <- 2 \%*\% x[3]
                
                cjac[1, 4] <- 2 \%*\% x[4]
                
            }
            if (needc[2] > 0) {
                
                cjac[2, 1] <- x[2] \%*\% x[3] \%*\% x[4]
                
                cjac[2, 2] <- x[1] \%*\% x[3] \%*\% x[4]
                
                cjac[2, 3] <- x[1] \%*\% x[2] \%*\% x[4]
                
                cjac[2, 4] <- x[1] \%*\% x[2] \%*\% x[3]
                
            }
        }
        ans <- e04uf(irevcm, nclin, a, bl, bu, iter, istate, 
            c, cjac, clamda, objf, objgrd, r, x, iwork, work, 
            cwsav, lwsav, iwsav, rwsav, optlist)
        irevcm <- ans$IREVCM
        iter <- ans$ITER
        istate <- ans$ISTATE
        c <- ans$C
        cjac <- ans$CJAC
        clamda <- ans$CLAMDA
        objf <- ans$OBJF
        objgrd <- ans$OBJGRD
        r <- ans$R
        x <- ans$X
        needc <- ans$NEEDC
        iwork <- ans$IWORK
        work <- ans$WORK
        cwsav <- ans$CWSAV
        lwsav <- ans$LWSAV
        iwsav <- ans$IWSAV
        rwsav <- ans$RWSAV
        ifail <- ans$IFAIL
    }
    if (ifail == 0) {
        
        writeLines(toString(cat(sprintf("\n Varbl Istate Value Lagr Mult\n", 
            "\n"))))
        
        
        for (i in c(1:4)) {
            istate <- ans$ISTATE
            
            x <- ans$X
            
            clamda <- ans$CLAMDA
            
            writeLines(toString(cat(sprintf(" V \%3d \%3d \%14.4f \%12.4f \n", 
                i, istate[i], x[i], clamda[i], "\n"))))
            
        }
        
        ax <- a \%*\% x
        writeLines(toString(cat(sprintf("\n L Con Istate Value Lagr Mult\n", 
            "\n"))))
        
        
        for (i in c(5:(4 + nclin))) {
            j <- i - 4
            
            istate <- ans$ISTATE
            
            
            clamda <- ans$CLAMDA
            writeLines(toString(cat(sprintf(" L \%3d \%3d \%14.4f \%12.4f\n", 
                j, istate[i], ax[j], clamda[i], "\n"))))
            
        }
        writeLines(toString(cat(sprintf("\n L Con Istate Value Lagr Mult\n", 
            "\n"))))
        
        
        for (i in c((5 + nclin):(6 + nclin))) {
            j <- i - 4 - nclin
            
            istate <- ans$ISTATE
            
            c <- ans$C
            
            clamda <- ans$CLAMDA
            
            writeLines(toString(cat(sprintf(" N \%3d \%3d \%14.4f\%12.4f\n", 
                j, istate[i], c[j], clamda[i], "\n"))))
            
        }
        objf <- ans$OBJF
        
        writeLines(toString(cat(sprintf("\n Final objective value = \%15.7f\n", 
            objf, "\n"))))
        
        
    }
} 

}

\keyword{math}

\keyword{optimize}
\concept{constraint}
\concept{derivative}
\concept{minimum}
\concept{nonlinear}
\concept{QP}
\concept{sequential}
