
\name{s18de}
\alias{s18de}

\title{s18de: Modified Bessel functions I_nu + a(z), real a  >=  0, complex z, nu = 0 , 1 , 2 ,  . . . }

\description{
s18de returns a sequence of values for the modified Bessel functions \eqn{{I}_{{{\nu} 
+ {n}}}\left({z}\right)}{I_\nu + n(z)} for complex \eqn{{z}}{z}, non-negative 
\eqn{{\nu}}{\nu} and \eqn{{n} = {0} , {1} , \ldots , {N} - {1}}{n = 0 , 1 ,  . . .  , N - 1}, 
with an option for exponential scaling.

}

\usage{
s18de(fnu, z, n, scal)
}

\arguments{

\item{fnu}{double

\eqn{{\nu}}{\nu}, the order of the first member of the sequence of functions.


}
\item{z}{complex

The argument \eqn{{z}}{z} of the functions.


}
\item{n}{integer

\eqn{{N}}{N}, the number of members required in the sequence \eqn{{I}_{{\nu}}\left({z}\right) 
, {I}_{{{\nu} + {1}}}\left({z}\right) , \ldots , {I}_{{{\nu} + {N} - {1}}}\left({z}\right)}{I_\nu(z) , I_\nu + 1(z) ,  . . .  , I_\nu + N - 1(z)}.


}
\item{scal}{string

The scaling option. 

\eqn{scal=\mathtt{'U'}}{scal='U'}: The results are returned unscaled.

\eqn{scal=\mathtt{'S'}}{scal='S'}: The results are returned scaled by the 
factor \eqn{{e}^{{ - \mathrm{abs}(\mathrm{Re}\left({z}\right))}}}{e^ - abs(Re(z))}.




}
}

\details{
R interface to the NAG Fortran routine S18DEF.}

\value{

\item{CY}{complex array

The \eqn{{N}}{N} required function values: \eqn{cy[{i}]}{cy[i]} contains 
\eqn{{I}_{{{\nu} + {i} - {1}}}\left({z}\right)}{I_\nu + i - 1(z)} for \eqn{{i}={1} 
\ldots {N}}{i=1  . . .  N}.


}
\item{NZ}{integer

The number of components of cy that are set to zero due to underflow.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/S/s18def.pdf}
}

\author{NAG}

\examples{

ifail<-0

fnu<-0

z<-complex(1,0.3,-0.4)

n<-2

scal<-'U'

s18de(fnu,z,n,scal)

}

\keyword{math}
\concept{Bessel}
\concept{modified}
