
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04bba_FUNCT(CONST double *XC, double *FC, double *GC,
                                  Integer *E04BBF_IUSER,
                                  double E04BBF_RUSER[]);

#ifdef __cplusplus
}
#endif

/*Declaration of global IUSER for this routine*/
Integer E04BBF_IUSER[4];

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04BBA_clean_up(void);

SEXP WE04BBF(SEXP FUNCT, SEXP E1, SEXP E2, SEXP A, SEXP B, SEXP MAXCAL,
             SEXP RHO)
{
  Integer IFAIL_integer;
  Integer E04BBF_IUSER[4];
  double  E04BBF_RUSER[1];
  SEXP    ans, names;

/*initialization for input variables*/
  double  E1_double;
  if (nag_extractDouble(E1, &E1_double))
    {
      error("Double extraction error for %s", "E1");
    }
  double  E2_double;
  if (nag_extractDouble(E2, &E2_double))
    {
      error("Double extraction error for %s", "E2");
    }
  double  A_double;
  if (nag_extractDouble(A, &A_double))
    {
      error("Double extraction error for %s", "A");
    }
  double  B_double;
  if (nag_extractDouble(B, &B_double))
    {
      error("Double extraction error for %s", "B");
    }
  int     MAXCAL_integer;
  if (nag_extractInteger(MAXCAL, &MAXCAL_integer))
    {
      error("Integer extraction error for %s", "MAXCAL");
    }

/*initialization for output variables*/
  double  X_double;
  double  F_double;
  double  G_double;

/*allocation general variables*/

/*extraction for input variables*/

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Get the maximum number of iterations, or use default of 100 if not supplied */
  if (MAXCAL_integer == NA_INTEGER) MAXCAL_integer = 100;
  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04BBF_IUSER[0]);
  nag_convert_p2i(FUNCT, &E04BBF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04bba_(e04bba_FUNCT, &E1_double, &E2_double, &A_double, &B_double,
          &MAXCAL_integer, &X_double, &F_double, &G_double, E04BBF_IUSER,
          E04BBF_RUSER,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04BBF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 9));
  SET_VECTOR_ELT(ans, 0, nag_d2r(1, &E1_double));
  SET_VECTOR_ELT(ans, 1, nag_d2r(1, &E2_double));
  SET_VECTOR_ELT(ans, 2, nag_d2r(1, &A_double));
  SET_VECTOR_ELT(ans, 3, nag_d2r(1, &B_double));
  SET_VECTOR_ELT(ans, 4, nag_i2r(1, &MAXCAL_integer));
  SET_VECTOR_ELT(ans, 5, nag_d2r(1, &X_double));
  SET_VECTOR_ELT(ans, 6, nag_d2r(1, &F_double));
  SET_VECTOR_ELT(ans, 7, nag_d2r(1, &G_double));
  SET_VECTOR_ELT(ans, 8, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 9));
  SET_STRING_ELT(names, 0, mkChar("E1"));
  SET_STRING_ELT(names, 1, mkChar("E2"));
  SET_STRING_ELT(names, 2, mkChar("A"));
  SET_STRING_ELT(names, 3, mkChar("B"));
  SET_STRING_ELT(names, 4, mkChar("MAXCAL"));
  SET_STRING_ELT(names, 5, mkChar("X"));
  SET_STRING_ELT(names, 6, mkChar("F"));
  SET_STRING_ELT(names, 7, mkChar("G"));
  SET_STRING_ELT(names, 8, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04BBA_clean_up();

  return(ans);
}

void NAG_CALL e04bba_FUNCT(CONST double *XC, double *FC, double *GC,
                           Integer *E04BBF_IUSER,
                           double E04BBF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04BBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04BBF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang2(
            funct,
            nag_d2r(1, XC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleScalarFromList("FC", result, FC);
  (void) nag_getDoubleScalarFromList("GC", result, GC);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04BBA_clean_up(void)
{
  return;
}
