
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04kyf_FUNCT2(CONST Integer *N, CONST double XC[],
                                   double *FC, double GC[],
                                   Integer *E04KYF_IUSER,
                                   double E04KYF_RUSER[]);

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04KYF_clean_up(double *BL, double *BU, double *X, double *G, Integer *IW,
                     double *W);

SEXP WE04KYF(SEXP N, SEXP IBOUND, SEXP FUNCT2, SEXP BL, SEXP BU, SEXP X,
             SEXP LIW, SEXP LW,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04KYF_IUSER[4];
  double  E04KYF_RUSER[1];
  SEXP    ans, names;

/*initialization for input variables*/
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int     IBOUND_integer;
  if (nag_extractInteger(IBOUND, &IBOUND_integer))
    {
      error("Integer extraction error for %s", "IBOUND");
    }
  double  *BL_doubleMatrix = 0;
  double  *BU_doubleMatrix = 0;
  double  *X_doubleMatrix = 0;
  int     LIW_integer;
  if (nag_extractInteger(LIW, &LIW_integer))
    {
      error("Integer extraction error for %s", "LIW");
    }
  int     LW_integer;
  if (nag_extractInteger(LW, &LW_integer))
    {
      error("Integer extraction error for %s", "LW");
    }

/*initialization for output variables*/
  double  F_double;
  double  *G_doubleMatrix = 0;
  Integer *IW_integerMatrix = 0;
  double  *W_doubleMatrix = 0;

/*allocation general variables*/
  Integer BL_dim1 = N_integer;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BL");
    }
  Integer BU_dim1 = N_integer;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BU");
    }
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer G_dim1 = N_integer;
  if (!(G_doubleMatrix = (double *) malloc(G_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "G");
    }
  Integer IW_dim1 = LIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer W_dim1 = LW_integer;
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "W");
    }

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BL_dim1, 1, BL, BL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BL", nrows(BL), BL_dim1);
        }
      else
        {
          E04KYF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                          G_doubleMatrix, IW_integerMatrix,
                          W_doubleMatrix);
          error("Array extraction error for %s", "BL");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BU_dim1, 1, BU, BU_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BU", nrows(BU), BU_dim1);
        }
      else
        {
          E04KYF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                          G_doubleMatrix, IW_integerMatrix,
                          W_doubleMatrix);
          error("Array extraction error for %s", "BU");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04KYF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                          G_doubleMatrix, IW_integerMatrix,
                          W_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04KYF_IUSER[0]);
  nag_convert_p2i(FUNCT2, &E04KYF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04kyf_(&N_integer, &IBOUND_integer, e04kyf_FUNCT2, BL_doubleMatrix,
          BU_doubleMatrix, X_doubleMatrix, &F_double, G_doubleMatrix,
          IW_integerMatrix,
          &LIW_integer, W_doubleMatrix, &LW_integer, E04KYF_IUSER,
          E04KYF_RUSER,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04KYF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 8));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, BL_dim1, 1, BL_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, BU_dim1, 1, BU_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_d2r(1, &F_double));
  SET_VECTOR_ELT(ans, 4, nag_md2r(1, G_dim1, 1, G_doubleMatrix));
  SET_VECTOR_ELT(ans, 5, nag_mi2r(1, IW_dim1, 1, IW_integerMatrix));
  SET_VECTOR_ELT(ans, 6, nag_md2r(1, W_dim1, 1, W_doubleMatrix));
  SET_VECTOR_ELT(ans, 7, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 8));
  SET_STRING_ELT(names, 0, mkChar("BL"));
  SET_STRING_ELT(names, 1, mkChar("BU"));
  SET_STRING_ELT(names, 2, mkChar("X"));
  SET_STRING_ELT(names, 3, mkChar("F"));
  SET_STRING_ELT(names, 4, mkChar("G"));
  SET_STRING_ELT(names, 5, mkChar("IW"));
  SET_STRING_ELT(names, 6, mkChar("W"));
  SET_STRING_ELT(names, 7, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04KYF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                  G_doubleMatrix, IW_integerMatrix,
                  W_doubleMatrix);

  return(ans);
}

void NAG_CALL e04kyf_FUNCT2(CONST Integer *N, CONST double XC[], double *FC,
                            double GC[], Integer *E04KYF_IUSER,
                            double E04KYF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04KYF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04KYF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang3(
            funct,
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleScalarFromList("FC", result, FC);
  (void) nag_getDoubleMatrixFromList("GC", 1, *N, 1, result, GC);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04KYF_clean_up(double *BL, double *BU, double *X, double *G, Integer *IW,
                     double *W)
{
  if (BL) free(BL);
  if (BU) free(BU);
  if (X) free(X);
  if (G) free(G);
  if (IW) free(IW);
  if (W) free(W);
  return;
}
