
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04nqf_QPHX(CONST Integer *NCOLH, CONST double X[],
                                 double HX[], CONST Integer *NSTATE,
                                 char CUSER[],
#if defined (_WIN32) && defined (USE_STDCALL)
                                 CONST int CUSER_dim,
#endif
                                 Integer *E04NQF_IUSER,
                                 double E04NQF_RUSER[]

#if defined (_WIN32) && defined (USE_STDCALL)
#else
                                , CONST int CUSER_dim
#endif
                                 );
#ifdef __cplusplus
}
#endif
int wE04NQF_handle_options(SEXP OPTLIST, char CW_charArray[],
                           Integer IW_integerMatrix[],
                           double RW_doubleMatrix[],
                           CONST int CW_dim)
;

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04NQF_clean_up(double *ACOL, Integer *INDA, Integer *LOCA, double *BL,
                     double *BU, double *C, char *NAMES, Integer *HELAST,
                     Integer *HS, double *X,
                     double *PI_NAG, double *RC, char *CW, Integer *IW,
                     double *RW);

SEXP WE04NQF(SEXP START, SEXP QPHX, SEXP M, SEXP N, SEXP NE, SEXP NNAME,
             SEXP LENC, SEXP NCOLH, SEXP IOBJ, SEXP OBJADD, SEXP PROB,
             SEXP ACOL, SEXP INDA,
             SEXP LOCA, SEXP BL, SEXP BU, SEXP C, SEXP NAMES, SEXP HELAST,
             SEXP HS, SEXP X,
             SEXP NS, SEXP OPTLIST,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04NQF_IUSER[4];
  double  E04NQF_RUSER[1];
  char    CUSER_charArray[1];

  SEXP    ans, names;
  Integer nadv, iflag;

  /* Initialize the option arrays */
  Integer IFAIL_initializer_integer = 1;
  char    err_msg[LEN_ERR_MSG];

/*Declare dimension variables for option initializer*/
  Integer LENCW_integer = 600;
  Integer LENIW_integer = 600;
  Integer LENRW_integer = 600;

/*Declare the parameters for the option initializer*/
  char    *CW_charArray = 0;
  Integer *IW_integerMatrix = 0;
  double  *RW_doubleMatrix = 0;
  Integer CW_dim1 = LENCW_integer;
  if (!(CW_charArray = (char *) malloc(CW_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "CW");
    }
  Integer IW_dim1 = LENIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer RW_dim1 = LENRW_integer;
  if (!(RW_doubleMatrix = (double *) malloc(RW_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "RW");
    }

  nag_initializeIntegerMatrix(IW_dim1, 1, IW_integerMatrix);
  nag_initializeDoubleMatrix(RW_dim1, 1, RW_doubleMatrix);
  e04npf_(CW_charArray, &LENCW_integer, IW_integerMatrix, &LENIW_integer,
          RW_doubleMatrix, &LENRW_integer, &IFAIL_initializer_integer,
          8);

  if (IFAIL_initializer_integer != 0)
    {
      sprintf(err_msg, "E04NPF failed with IFAIL = %ld\n",
              (long) IFAIL_initializer_integer);
      error(err_msg);
    } /* Handle options */
  int     unit_num;

  unit_num =
    wE04NQF_handle_options(OPTLIST, CW_charArray, IW_integerMatrix,
                           RW_doubleMatrix,
                           8);
  if (unit_num < 0)
    {
      /* Error opening output file */
      sprintf(
        err_msg,
        "Error opening monitoring file for output, check value for `monitoring file` entry in `optlist`");
      error(err_msg);
    }
  else if (unit_num > 0)
    {
      /* We are using an output file, so need to handle the advisory channel */

      /* Remember the current value */
      iflag = 0;
      x04abf_(&iflag, &nadv);

      /* Set the advisory channel to the output file. Need to do this, otherwise E04NQF sends output to both the
         advisory channel and the output file */
      iflag = 1;
      x04abf_(&iflag, &unit_num);
    }

/*initialization for input variables*/
  const char *START_char = CHAR(STRING_ELT(START, 0));
  Integer    START_length = 1;
  int        M_integer;
  if (nag_extractInteger(M, &M_integer))
    {
      error("Integer extraction error for %s", "M");
    }
  int        N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int        NE_integer;
  if (nag_extractInteger(NE, &NE_integer))
    {
      error("Integer extraction error for %s", "NE");
    }
  int        NNAME_integer;
  if (nag_extractInteger(NNAME, &NNAME_integer))
    {
      error("Integer extraction error for %s", "NNAME");
    }
  int        LENC_integer;
  if (nag_extractInteger(LENC, &LENC_integer))
    {
      error("Integer extraction error for %s", "LENC");
    }
  int        NCOLH_integer;
  if (nag_extractInteger(NCOLH, &NCOLH_integer))
    {
      error("Integer extraction error for %s", "NCOLH");
    }
  int        IOBJ_integer;
  if (nag_extractInteger(IOBJ, &IOBJ_integer))
    {
      error("Integer extraction error for %s", "IOBJ");
    }
  double     OBJADD_double;
  if (nag_extractDouble(OBJADD, &OBJADD_double))
    {
      error("Double extraction error for %s", "OBJADD");
    }
  const char *PROB_char = CHAR(STRING_ELT(PROB, 0));
  Integer    PROB_length = 1;
  double     *ACOL_doubleMatrix = 0;
  Integer    *INDA_integerMatrix = 0;
  Integer    *LOCA_integerMatrix = 0;
  double     *BL_doubleMatrix = 0;
  double     *BU_doubleMatrix = 0;
  double     *C_doubleMatrix = 0;
  char       *NAMES_charArray = 0;
  Integer    *HELAST_integerMatrix = 0;
  Integer    *HS_integerMatrix = 0;
  double     *X_doubleMatrix = 0;
  int        NS_integer;
  if (nag_extractInteger(NS, &NS_integer))
    {
      error("Integer extraction error for %s", "NS");
    }

/*initialization for output variables*/
  double     *PI_NAG_doubleMatrix = 0;
  double     *RC_doubleMatrix = 0;
  int        NINF_integer;
  double     SINF_double;
  double     OBJ_double;

/*allocation general variables*/
  Integer    ACOL_dim1 = NE_integer;
  if (!(ACOL_doubleMatrix = (double *) malloc(ACOL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "ACOL");
    }
  Integer    INDA_dim1 = NE_integer;
  if (!(INDA_integerMatrix = (Integer *) malloc(INDA_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "INDA");
    }
  Integer    LOCA_dim1 = N_integer+1;
  if (!(LOCA_integerMatrix = (Integer *) malloc(LOCA_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "LOCA");
    }
  Integer    BL_dim1 = N_integer+M_integer;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BL");
    }
  Integer    BU_dim1 = N_integer+M_integer;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BU");
    }
  Integer    C_dim1;
  if (IOBJ_integer != 0)
    {
      C_dim1 = MAX(1, LENC_integer);
    }
  else
    {
      C_dim1 = 1;
    }
  if (!(C_doubleMatrix = (double *) malloc(C_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "C");
    }
  Integer    NAMES_dim1 = NNAME_integer;
  if (!(NAMES_charArray = (char *) malloc(NAMES_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "NAMES");
    }
  Integer    HELAST_dim1 = N_integer+M_integer;
  if (!(HELAST_integerMatrix = (Integer *) malloc(HELAST_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "HELAST");
    }
  Integer    HS_dim1 = N_integer+M_integer;
  if (!(HS_integerMatrix = (Integer *) malloc(HS_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "HS");
    }
  Integer    X_dim1 = N_integer+M_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer    PI_NAG_dim1 = M_integer;
  if (!(PI_NAG_doubleMatrix = (double *) malloc(PI_NAG_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "PI_NAG");
    }
  Integer    RC_dim1 = N_integer+M_integer;
  if (!(RC_doubleMatrix = (double *) malloc(RC_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "RC");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, ACOL_dim1, 1, ACOL, ACOL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "ACOL", nrows(ACOL), ACOL_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "ACOL");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, INDA_dim1, 1, INDA, INDA_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "INDA", nrows(INDA), INDA_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "INDA");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, LOCA_dim1, 1, LOCA, LOCA_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "LOCA", nrows(LOCA), LOCA_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "LOCA");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BL_dim1, 1, BL, BL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BL", nrows(BL), BL_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "BL");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BU_dim1, 1, BU, BU_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BU", nrows(BU), BU_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "BU");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, C_dim1, 1, C, C_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "C", nrows(C), C_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "C");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractCharVector(NAMES_dim1, NAMES, NAMES_charArray, 8)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "NAMES", nrows(NAMES), NAMES_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "NAMES");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, HELAST_dim1, 1, HELAST,
                                  HELAST_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "HELAST", nrows(HELAST), HELAST_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "HELAST");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, HS_dim1, 1, HS, HS_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "HS", nrows(HS), HS_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "HS");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix,
                          LOCA_integerMatrix, BL_doubleMatrix, BU_doubleMatrix,
                          C_doubleMatrix,
                          NAMES_charArray, HELAST_integerMatrix,
                          HS_integerMatrix, X_doubleMatrix,
                          PI_NAG_doubleMatrix, RC_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04NQF_IUSER[0]);
  nag_convert_p2i(QPHX, &E04NQF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04nqf_(START_char, e04nqf_QPHX, &M_integer, &N_integer, &NE_integer,
          &NNAME_integer, &LENC_integer, &NCOLH_integer, &IOBJ_integer,
          &OBJADD_double,
          PROB_char, ACOL_doubleMatrix, INDA_integerMatrix, LOCA_integerMatrix,
          BL_doubleMatrix, BU_doubleMatrix, C_doubleMatrix, NAMES_charArray,
          HELAST_integerMatrix, HS_integerMatrix, X_doubleMatrix,
          PI_NAG_doubleMatrix,
          RC_doubleMatrix, &NS_integer, &NINF_integer, &SINF_double,
          &OBJ_double,
          CW_charArray, &LENCW_integer, IW_integerMatrix, &LENIW_integer,
          RW_doubleMatrix, &LENRW_integer, CUSER_charArray, E04NQF_IUSER,
          E04NQF_RUSER,
          &IFAIL_integer, START_length, PROB_length, 8, 8,
          8);

  /* Close the monitoring file if one opened */

  if (unit_num > 0)
    {
      IFAIL_integer = 1;
      x04adf_(&unit_num, &IFAIL_integer);

      /* Set the advisory channel back to its previous value */
      iflag = 1;
      x04abf_(&iflag, &nadv);
    }

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04NQF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 9));
  SET_VECTOR_ELT(ans, 0, nag_mi2r(1, HS_dim1, 1, HS_integerMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, PI_NAG_dim1, 1, PI_NAG_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_md2r(1, RC_dim1, 1, RC_doubleMatrix));
  SET_VECTOR_ELT(ans, 4, nag_i2r(1, &NS_integer));
  SET_VECTOR_ELT(ans, 5, nag_i2r(1, &NINF_integer));
  SET_VECTOR_ELT(ans, 6, nag_d2r(1, &SINF_double));
  SET_VECTOR_ELT(ans, 7, nag_d2r(1, &OBJ_double));
  SET_VECTOR_ELT(ans, 8, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 9));
  SET_STRING_ELT(names, 0, mkChar("HS"));
  SET_STRING_ELT(names, 1, mkChar("X"));
  SET_STRING_ELT(names, 2, mkChar("PI_NAG"));
  SET_STRING_ELT(names, 3, mkChar("RC"));
  SET_STRING_ELT(names, 4, mkChar("NS"));
  SET_STRING_ELT(names, 5, mkChar("NINF"));
  SET_STRING_ELT(names, 6, mkChar("SINF"));
  SET_STRING_ELT(names, 7, mkChar("OBJ"));
  SET_STRING_ELT(names, 8, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04NQF_clean_up(ACOL_doubleMatrix, INDA_integerMatrix, LOCA_integerMatrix,
                  BL_doubleMatrix, BU_doubleMatrix, C_doubleMatrix,
                  NAMES_charArray,
                  HELAST_integerMatrix, HS_integerMatrix, X_doubleMatrix,
                  PI_NAG_doubleMatrix,
                  RC_doubleMatrix, CW_charArray, IW_integerMatrix,
                  RW_doubleMatrix);

  return(ans);
}

void NAG_CALL e04nqf_QPHX(CONST Integer *NCOLH, CONST double X[], double HX[],
                          CONST Integer *NSTATE, char CUSER[],
#if defined (_WIN32) && defined (USE_STDCALL)
                          CONST int CUSER_dim,
#endif
                          Integer *E04NQF_IUSER,
                          double E04NQF_RUSER[]
#if defined (_WIN32) && defined (USE_STDCALL)
#else
                         , CONST int CUSER_dim
#endif
                          )
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04NQF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04NQF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang4(
            funct,
            nag_fi2r(1, NCOLH),
            nag_md2r(1, *NCOLH, 1, X),
            nag_fi2r(1, NSTATE)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleMatrixFromList("HX", 1, *NCOLH, 1, result, HX);

/* Release R_fcall */
  UNPROTECT(2);
}

int wE04NQF_handle_options(SEXP OPTLIST, char CW_charArray[],
                           Integer IW_integerMatrix[],
                           double RW_doubleMatrix[],
                           CONST int CW_dim)
{
  SEXP       names, tentry;
  char       err_msg[LEN_ERR_MSG];
  int        i, ival;
  Integer    IFAIL_setter_integer, STRING_length;
  double     dval;
  const char *this_name;
  char       STRING_char[100];
  int        unit_num = 0;

  /* No options specified */
  if (!isNewList(OPTLIST))
    {
      return 0;
    }

  /* Get the list of names from the list */
  PROTECT(names = getAttrib(OPTLIST, R_NamesSymbol));
  for (i = 0; i < length(OPTLIST); i++)
    {
      this_name = CHAR(STRING_ELT(names, i));

      /* Only interested in the first element, which is what asReal should
         convert to a double */
      PROTECT(tentry = VECTOR_ELT(OPTLIST, i));
      if (!isString(tentry))
        {
          dval = asReal(tentry);
          if (!R_FINITE(dval)) dval = 0.0;
          ival = (int) dval;
        }
      else
        {
          dval = -1.0;
          ival = -1;
        }

      STRING_char[0] = '\0';
      if (!strcmp(this_name, "list"))
        {
          if (ival)
            {
              sprintf(STRING_char, "list");
            }
          else
            {
              sprintf(STRING_char, "nolist");
            }
        }
      else if (!strcmp(this_name, "Check Frequency"))
        {
          sprintf(STRING_char, "Check Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Option"))
        {
          sprintf(STRING_char, "Crash Option = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Tolerance"))
        {
          sprintf(STRING_char, "Crash Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Elastic Mode"))
        {
          sprintf(STRING_char, "Elastic Mode = %d", ival);
        }
      else if (!strcmp(this_name, "Elastic Objective"))
        {
          sprintf(STRING_char, "Elastic Objective = %d", ival);
        }
      else if (!strcmp(this_name, "Elastic Weight"))
        {
          sprintf(STRING_char, "Elastic Weight = %g", dval);
        }
      else if (!strcmp(this_name, "Expand Frequency"))
        {
          sprintf(STRING_char, "Expand Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Factorization Frequency"))
        {
          sprintf(STRING_char, "Factorization Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Feasibility Tolerance"))
        {
          sprintf(STRING_char, "Feasibility Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Infinite Bound Size"))
        {
          sprintf(STRING_char, "Infinite Bound Size = %g", dval);
        }
      else if (!strcmp(this_name, "Iterations Limit"))
        {
          sprintf(STRING_char, "Iterations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Old Basis File"))
        {
          sprintf(STRING_char, "Old Basis File = %d", ival);
        }
      else if (!strcmp(this_name, "Optimality Tolerance"))
        {
          sprintf(STRING_char, "Optimality Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Partial Price"))
        {
          sprintf(STRING_char, "Partial Price = %d", ival);
        }
      else if (!strcmp(this_name, "Pivot Tolerance"))
        {
          sprintf(STRING_char, "Pivot Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Print File"))
        {
          sprintf(STRING_char, "Print File = %d", ival);
        }
      else if (!strcmp(this_name, "Print Frequency"))
        {
          sprintf(STRING_char, "Print Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Print Level"))
        {
          sprintf(STRING_char, "Print Level = %d", ival);
        }
      else if (!strcmp(this_name, "Reduced Hessian Dimension"))
        {
          sprintf(STRING_char, "Reduced Hessian Dimension = %d", ival);
        }
      else if (!strcmp(this_name, "Scale Option"))
        {
          sprintf(STRING_char, "Scale Option = %d", ival);
        }
      else if (!strcmp(this_name, "Scale Tolerance"))
        {
          sprintf(STRING_char, "Scale Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Solution File"))
        {
          sprintf(STRING_char, "Solution File = %d", ival);
        }
      else if (!strcmp(this_name, "Superbasics Limit"))
        {
          sprintf(STRING_char, "Superbasics Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Timing Level"))
        {
          sprintf(STRING_char, "Timing Level = %d", ival);
        }
      else if (!strcmp(this_name, "Unbounded Step Size"))
        {
          sprintf(STRING_char, "Unbounded Step Size = %g", dval);
        }

      /* All other options are ignored (this includes the valid ones: cold start and warm start) */

      UNPROTECT(1);

      /* Call the option setting routine */
      if (STRING_char[0])
        {
          IFAIL_setter_integer = 1;
          STRING_length = strlen(STRING_char);
          e04nsf_(STRING_char, CW_charArray, IW_integerMatrix, RW_doubleMatrix,
                  &IFAIL_setter_integer, STRING_length,
                  8);

          if (IFAIL_setter_integer != 0)
            {
              sprintf(err_msg, "E04NSF failed with IFAIL = %ld\n",
                      (long) IFAIL_setter_integer);
              error(err_msg);
            }
        }
    }
  UNPROTECT(1);
  return unit_num;
}

void E04NQF_clean_up(double *ACOL, Integer *INDA, Integer *LOCA, double *BL,
                     double *BU, double *C, char *NAMES, Integer *HELAST,
                     Integer *HS, double *X,
                     double *PI_NAG, double *RC, char *CW, Integer *IW,
                     double *RW)
{
  if (ACOL) free(ACOL);
  if (INDA) free(INDA);
  if (LOCA) free(LOCA);
  if (BL) free(BL);
  if (BU) free(BU);
  if (C) free(C);
  if (NAMES) free(NAMES);
  if (HELAST) free(HELAST);
  if (HS) free(HS);
  if (X) free(X);
  if (PI_NAG) free(PI_NAG);
  if (RC) free(RC);
  if (CW) free(CW);
  if (IW) free(IW);
  if (RW) free(RW);
  return;
}
