
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void G02AAF_clean_up(double *G, double *X);

SEXP WG02AAF(SEXP G, SEXP N, SEXP ERRTOL, SEXP MAXITS, SEXP MAXIT)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  double  *G_doubleMatrix = 0;
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  double  ERRTOL_double;
  if (nag_extractDouble(ERRTOL, &ERRTOL_double))
    {
      error("Double extraction error for %s", "ERRTOL");
    }
  int     MAXITS_integer;
  if (nag_extractInteger(MAXITS, &MAXITS_integer))
    {
      error("Integer extraction error for %s", "MAXITS");
    }
  int     MAXIT_integer;
  if (nag_extractInteger(MAXIT, &MAXIT_integer))
    {
      error("Integer extraction error for %s", "MAXIT");
    }

/*initialization for dimension variables*/
  Integer LDG_integer = N_integer;
  Integer LDX_integer = N_integer;

/*initialization for output variables*/
  double  *X_doubleMatrix = 0;
  int     ITER_integer;
  int     FEVAL_integer;
  double  NRMGRD_double;

/*allocation general variables*/
  Integer G_dim1 = LDG_integer;
  Integer G_dim2 = N_integer;
  if (!(G_doubleMatrix = (double *) malloc(G_dim1*G_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "G");
    }
  Integer X_dim1 = LDX_integer;
  Integer X_dim2 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*X_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, G_dim1, G_dim2, G, G_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "G", nrows(G), G_dim1);
        }
      else
        {
          G02AAF_clean_up(G_doubleMatrix, X_doubleMatrix);
          error("Array extraction error for %s", "G");
        }
    }

  /* Call the NAG routine */
  IFAIL_integer = 1;
  g02aaf_(G_doubleMatrix, &LDG_integer, &N_integer, &ERRTOL_double,
          &MAXITS_integer, &MAXIT_integer, X_doubleMatrix, &LDX_integer,
          &ITER_integer,
          &FEVAL_integer, &NRMGRD_double,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  G02AAF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 6));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, G_dim1, G_dim2, G_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, X_dim1, X_dim2, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &ITER_integer));
  SET_VECTOR_ELT(ans, 3, nag_i2r(1, &FEVAL_integer));
  SET_VECTOR_ELT(ans, 4, nag_d2r(1, &NRMGRD_double));
  SET_VECTOR_ELT(ans, 5, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 6));
  SET_STRING_ELT(names, 0, mkChar("G"));
  SET_STRING_ELT(names, 1, mkChar("X"));
  SET_STRING_ELT(names, 2, mkChar("ITER"));
  SET_STRING_ELT(names, 3, mkChar("FEVAL"));
  SET_STRING_ELT(names, 4, mkChar("NRMGRD"));
  SET_STRING_ELT(names, 5, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  G02AAF_clean_up(G_doubleMatrix, X_doubleMatrix);

  return(ans);
}

void G02AAF_clean_up(double *G, double *X)
{
  if (G) free(G);
  if (X) free(X);
  return;
}
